from datetime import datetime

from pydantic import BaseModel, PrivateAttr, computed_field

from bigdata_client.connection_protocol import BigdataConnectionProtocol


class ChatInteraction(BaseModel):
    question: str
    answer: str
    interaction_timestamp: str
    date_created: datetime
    last_updated: datetime


class Chat(BaseModel):
    id: str
    name: str
    user_id: str
    date_created: datetime
    last_updated: datetime

    @computed_field
    @property
    def interactions(self) -> list[ChatInteraction]:
        if not self._loaded:
            self._reload()
        return self._interactions

    _api_connection: BigdataConnectionProtocol = PrivateAttr()
    _interactions: list[ChatInteraction] = PrivateAttr()
    _loaded: bool = PrivateAttr()

    def __init__(self, **values):
        super().__init__(**values)
        if "_api_connection" not in values:
            raise ValueError("Creating a chat requires an API connection")
        self._api_connection = values["_api_connection"]

        if "_interactions" in values and values["_interactions"] is not None:
            self._interactions = values["_interactions"]

        if "_loaded" in values:
            self._loaded = values["_loaded"]

    def ask(self, question: str) -> ChatInteraction:
        pass

    def _reload(self):
        chat = self._api_connection.get_chat(self.id).to_chat_model(
            self._api_connection
        )
        self.name = chat.name
        self.user_id = chat.user_id
        self.date_created = chat.date_created
        self.last_updated = chat.last_updated
        self._interactions = chat._interactions
        self._loaded = True

    def delete(self):
        self._api_connection.delete_chat(self.id)
